# Relaxation Method
# vertical position of a thrown ball
# d^2x/dt^2=-g
# b.c x=0 at time t=0 and t=10
# set dx/dt=v then dv/dt=-g
import numpy as np
import matplotlib.pyplot as plt

def f(r, t):
  global G
  return -G

def RK4(f, x, t, h):
  c=np.zeros((4, 2), float)
  c[0]=h*f(x, t)
  c[1]=h*f(x+c[0]/2, t+h/2)
  c[2]=h*f(x+c[1]/2, t+h/2)
  c[3]=h*f(x+c[2], t+h)
  return x+(c[0]+2*c[1]+2*c[2]+c[3])/6

if __name__=="__main__":
  G=9.81
  t_i=0.; t_f=10.
  t=np.linspace(t_i, t_f, 100)
  h=t[1]-t[0]
  
  x=np.zeros_like(t, float)
  x[0]=x[-1]=0.
  x_tmp=np.zeros_like(x, float)
  delta=1.
  tolerance=1e-5
  n=0
  
  while delta>tolerance:
    n+=1
    for k in range(1, len(t)-1):    # boundary k=0, k=len(t)-1 포함 X
      x_tmp[k]=(x[k+1]+x[k-1]-h**2*f(x[k], t[k]))/2
    delta=max(abs(x-x_tmp))
    x, x_tmp=x_tmp, x
  
  print("TRY: ", n, sep="")
  plt.plot(t, x)
  plt.show()
